#!/usr/local/bin/python2.5

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================


"""This is the main file to access to astk services through as_run
"""

__revision__ = "$Id $"

import os
import sys

# ----- differ messages translation
def _(mesg):
   return mesg

# ----- get ASTK_SERV location
main_py = sys.argv.pop(0)
if os.path.islink(main_py):
   main_py = os.path.realpath(main_py)
astk_serv_root = os.path.normpath(os.path.join(
      os.path.dirname(os.path.abspath(main_py)), os.pardir))

# ----- verification de la version de Python
if sys.hexversion < 0x020400F0:
   print _("This script requires Python 2.4 or higher, sorry !")
   sys.exit(4)

# ----- modules location
sys.path.insert(0, os.path.join(astk_serv_root, 'lib'))

# ----- import as_run modules
from as_run       import ASTER_RUN
from as_system    import ASTER_SYSTEM

#-------------------------------------------------------------------------------
def main():
   # ----- divers
   os.stat_float_times(True)

   # ----- initialisation
   run = ASTER_RUN(astk_serv_root)

   # ----- backward compatibility
   from backward_compatibility import change_argv
   sys.argv = change_argv(run, sys.argv)

   # ----- retrieve options and arguments
   opts, args = run.ParseArgs()
   run.current_action = opts.action

   if run.current_action == None:
      # if symbolic link "action" -> "as_run --action"
      alias = os.path.basename(sys.argv[0])
      if alias in run.actions_info.keys():
   	 run.current_action = alias
      else:
   	 # default to 'run'
   	 run.current_action = 'run'
   	 #run.parser.error(_('you must specify an action'))

   # ----- get system commands
   from backward_compatibility import get_parameter
   para = {
      'maxcmdlen'	: get_parameter("""value = 16384""", 1024),
      'multi_threading' : get_parameter("""value = True""", False),
   }
   run.system = ASTER_SYSTEM(run, **para)
   run.PostConf()

   # ----- debug information
   if run['debug']:
      run.PrintConfig()
      print os.linesep+_('Arguments')+' : '+repr(args)
      print

   # ----- start 'current_action'
   try :
      meth = run.actions_info[run.current_action]['method']
   except KeyError:
      run.Mess(_('dictionnary bad defined :')+' actions_info', _('<F>_PROGRAM_ERROR'))
   else:
      # trap <Control+C>
      try:
   	 meth(run, *args)
      except KeyboardInterrupt:
   	 run.Mess(_('%s stopped by user') % run.current_action, _('<F>_INTERRUPT'))

   run.Sortie(0)

if __name__ == '__main__':
   if len(sys.argv) > 1 and sys.argv[1] == '--pdb':
      del sys.argv[1]
      import pdb
      pdb.run('main()')
   else:
      main()
