# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Defines a function
"""

__revision__ = "$Id $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import sys
import time
import random
from sets import Set
from math import log10

from as_thread      import Dispatcher
from as_distrib     import DistribParametricTask
from as_common_func import get_hostrc, get_timeout

fmt_res0 = 'testcase     result                cpu      sys    cpu+sys  elapsed'
fmt_resu = '%-12s %-18s %8.2f %8.2f %8.2f %8.2f'
fmt_res2 = '%4d tests   %4d errors        %8.2f %8.2f %8.2f %8.2f'
fmt_tot = '-'*12+' '+'-'*18+' '+'-'*8+' '+'-'*8+' '+'-'*8+' '+'-'*8

#-------------------------------------------------------------------------------
def Parametric(run, prof, mpi_info, numthread='auto', **kargs):
   """Run a parametric study.
   """
   run.print_timer = True
   run.timer.Start(_('Parametric execution'), hide=True)

   prev = os.getcwd()

   # 1. ----- initializations
   jn = run['num_job']
   REPREF  = os.path.join(run.aster_root, prof['version'][0])

   # 1.2. rep_trav from profile or from run[...]
   reptrav = mpi_info.reptrav()
   
   run.Mess(_('Code_Aster parametric execution'), 'TITLE')
   mpi_info.set_cpuinfo(1, 1)
   
   # ----- how many threads ?
   if type(numthread) not in (int, long):
      numthread = run.GetCpuInfo('numthread')

   # 1.3. content of the profile
   if not prof.Get('D', typ='distr'):
      run.Mess(_('"distr" file is necessary'), _('<F>_NOT_FOUND'))
   else:
      fdist = prof.Get('D', typ='distr')[0]['path']
      if run.IsRemote(fdist):
         tmpdist = os.path.join(run['tmp_user'], 'distr.'+jn)
         run.ToDelete(tmpdist)
         kret = run.Copy(tmpdist, fdist)
         fdist = tmpdist
      else:
         fdist = run.PathOnly(fdist)
   dist_cnt = open(fdist, 'r').read()
   list_val = get_parameter_value(dist_cnt)
   if not is_list_of_dict(list_val):
      run.Mess(_('"distr" file must be defined a list of dicts'), _('<F>_INVALID'))
   nbval = len(list_val)

   if not prof.Get('R', typ='repe'):
      run.Mess(_('no result directory found (type "repe")'), ('<F>_NO_RESU_DIR'))
   else:
      resudir = prof.Get('R', typ='repe')[0]['path']
      run.MkDir(resudir)
      run.MkDir(os.path.join(resudir, 'flash'))
      
   # suivi des calculs partagé entre 'numthread' threads
   hostrc = get_hostrc(run, prof)
   sum_cpu = hostrc.get_sum('cpu')
   
   # timeout before rejected a job
   timeout = get_timeout(run, prof, 24.*3600.)
   
   # print a summary
   summary = _("""
--- %d calculations to run
--- Run started at %s
--- Parameters used for this run :
    Directory to copy results       : %s
    Code_Aster version              : %s
    Executable filename             : %s
    Commands catalogue directory    : %s
    Elements catalogue filename     : %s
    Working directory               : %s
    Submission timeout (seconds)    : %.0f""")
    
   sum_thread = _("""    Number of threads               : %d""")
   sum_rc     = _("""    Available hosts (number of cpu) : %s""")
   sum_end = _("""
--- All executions finished at %s

--- Results :
""")
   fmt_spup = _("""
--- Speed-up is %.2f
""")
   info_start = ( nbval, time.strftime('%c'), resudir,
      prof['version'][0], kargs['exe'], kargs['cmde'], kargs['ele'],
      reptrav, timeout )
   txt_summary = [ summary % info_start, ]
   if numthread > 1:
      txt_summary.append( sum_thread % numthread )
   if hostrc:
      host_str = ', '.join(['%(host)s (%(cpu)d)' % hostrc.GetConfig(h) for h in hostrc.all_hosts ])
      txt_summary.append( sum_rc % host_str )
   txt_summary.append('')
   text_summary = os.linesep.join(txt_summary)
   print text_summary
   
   # change directory before running sub-execution (reptrav will be deleted !)
   os.chdir(run['rep_trav'])
   
   # ----- Execute calcutions in parallel using a Dispatcher object
   # elementary task...
   task = DistribParametricTask(run=run, prof=prof, # IN
                          hostrc=hostrc,
                          nbmaxitem=0, timeout=timeout,
                          resudir=resudir,
                          info=1,
                          nbnook=0, exec_result=[])            # OUT
   # ... and dispatch task on 'list_tests'
   etiq = 'calc_%%0%dd' % (int(log10(nbval)) + 1)
   labels = [etiq % (i+1) for i in range(nbval)]
   couples = zip(labels, list_val)
   execution = Dispatcher(couples, task, numthread=numthread)
   
   # Summary
   tit = _('Summary of the run')
   run.Mess(tit, 'TITLE')
   print text_summary
   print sum_end % time.strftime('%c')
   t = [0., 0., 0., 0.]
   print fmt_res0
   print fmt_tot
   task.exec_result.sort()
   for result in task.exec_result:
      lin = result[:]
      del lin[1]
      t[0] += lin[2]
      t[1] += lin[3]
      t[2] += lin[4]
      t[3] += lin[5]
      print fmt_resu % tuple(lin)
   print fmt_tot
   print fmt_res2 % (len(task.exec_result), task.nbnook, t[0], t[1], t[2], t[3])
   
   # check the number of calculations really run
   if len(task.exec_result) != len(list_val):
      run.Mess(_('%d test-cases to run, %d really run !') \
         % (len(list_val), len(task.exec_result)), _('<E>_ERROR'))
   
   # force global diagnostic to <F>_ERROR if errors occured
   if task.nbnook > 0:
      run.diag = _('<E>_ERROR')

   cpu_dt, sys_dt, tot_dt = run.timer.StopAndGet(_('Parametric execution'))
   spup = 0.
   if tot_dt != 0.:
      spup = t[3]/tot_dt
   print fmt_spup % spup
   if hostrc is not None:
      print os.linesep + hostrc.repr_history()

   run.CheckOK()
   run.Mess(_('All calculations run successfully'), 'OK')
   os.chdir(prev)


#-------------------------------------------------------------------------------
def get_parameter_value(txt):
   """Retourne la liste des valeurs des parametres."""
   from N__F import _F
   d = locals().copy()
   key = Set(d.keys())
   exec txt in d
   del d['__builtins__']
   key.symmetric_difference_update(d.keys())
   if len(key) != 1:
      dico = None
   else:
      dico = d[key.pop()]
   return dico

#-------------------------------------------------------------------------------
def is_list_of_dict(obj):
   """Return True if obj is a list (iterable) of dictionnaries (with items method)."""
   try:
      for sobj in obj:
         for k, v in sobj.items():
            pass
      res = True
   except:
      res = False
   return res

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
if __name__ == '__main__':
   # unittest
   content = """VALE=(
   _F(P1=1., P2=0.3),
   _F(P1=2., P2=0.4),
)
"""
   dicval = get_parameter_value(content)
   assert is_list_of_dict(dicval)


