# -*- coding: utf-8 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""usage: as_run --serv [options] user@mach:/nom_profil.export

old syntax : as_serv as_exec [options] user@mach:/nom_profil.export

Export file must be the last argument.

Object :
   - retreive the export file from the client
   - read export and ...
   - ... call as_run on the right node, into a xterm or not,
     now or differed...
"""

__revision__ = "$Id: as_serv.py 3590 2008-11-06 13:56:43Z courtois $"


from as_calcul          import ASTER_CALCUL

# ----- differ messages translation
def _(mesg):
   return mesg


#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr = {
      'serv' : {
         'method' : Serv,
         'syntax' : 'user@mach:/nom_profil.export',
         'help'   : 'start an execution (calling as_run in a separate process)'
      },
   }
   run.SetActions(
         actions_descr = acts_descr,
         actions_order = ['serv',],
         group_options=False,
         options_descr = {},
   )

#-------------------------------------------------------------------------------
def Serv(run, *args):
   """Start an execution, another as_run, in a separate process.
   """
   # ----- check argument
   if len(args) != 1:
      run.parser.error(_("'--%s' requires one argument") % run.current_action)

   service = ASTER_CALCUL(run, filename=args[0])
   iret, output = service.start()
   # prinf job info
   print "JOBID=%s QUEUE=%s" % (service.jobid, service.queue)

   run.Sortie(iret)



