# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Give functions :
- to compile code source, to create catalogue and to build
an executable file,
- to execute Code_Aster...
Functions are called by an ASTER_RUN object.
"""

__revision__ = "$Id: old_astout.py 3668 2008-12-09 10:17:50Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import sys
import time
import re
from glob           import glob

from as_profil      import ASTER_PROFIL
from as_build       import ASTER_BUILD
from as_calcul      import ASTER_TESTCASE
from as_utils       import get_list, getpara
from as_execution   import execute, dict_typ_test

#-------------------------------------------------------------------------------
def RunAstout(run, conf, prof, mpi_info, numthread='auto', **kargs):
   """Run a list of test cases...
   """
   prev = os.getcwd()

   # 1. ----- initializations
   jn = run['num_job']
   REPREF  = os.path.join(run.aster_root, prof['version'][0])

   # 1.2. rep_trav from profile or from run[...]
   reptrav = mpi_info.reptrav()
   
   run.Mess(_('Code_Aster tests execution'), 'TITLE')
   mpi_info.set_cpuinfo(1, 1)

   os.chdir(reptrav)
      
   # 1.3. content of the profile
   ltest = os.path.join(reptrav, 'tests.list')
   if not prof.Get('D', typ='list'):
      run.Mess(_('no list of tests found'), _('<E>_NO_TEST_LIST'))
   else:
      for dli in prof.Get('D', typ='list'):
         if run.IsRemote(dli['path']):
            tmplist = os.path.join(run['tmp_user'], 'list.'+jn)
            run.ToDelete(tmplist)
            kret = run.Copy(tmplist, dli['path'])
            run.FileCat(tmplist, ltest)
         else:
            tmplist = run.PathOnly(dli['path'])
            run.FileCat(tmplist, ltest)
   if not prof.Get('D', typ='rep_test'):
      reptest = []
   else:
      reptest = [r['path'] for r in prof.Get('D', typ='rep_test')]
   if not prof.Get('R', typ='resu_test'):
      run.Mess(_('no result directory found'), ('<E>_NO_RESU_DIR'))
   else:
      resutest = prof.Get('R', typ='resu_test')[0]['path']
      run.MkDir(resutest)

   facmtps = 1.
   nbmaxnook = 5
   cpresok = 'RESNOOK'
   try:
      if prof['facmtps'][0] != '':
         facmtps   = float(prof['facmtps'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('facmtps', prof['facmtps'][0]))
   try:
      if prof['nbmaxnook'][0] != '':
         nbmaxnook = int(prof['nbmaxnook'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('nbmaxnook', prof['nbmaxnook'][0]))
   if prof['cpresok'][0] != '':
      cpresok   = prof['cpresok'][0]
   run.CheckOK()
   
   # 3.6.1. prepare reptrav to run Code_Aster
   build = ASTER_BUILD(run, conf)
   if prof.Get('D', typ='py'):
      kargs['py'] = [l['path'] for l in prof.Get('D', typ='py')]
   run.Mess(_('prepare environment in %s') % reptrav)
   run.timer.Start(_('Preparation of environment'))
   dbg = prof['debug'][0]
   build.PrepEnv(REPREF, reptrav, dbg=dbg, **kargs)
   run.timer.Stop(_('Preparation of environment'))

   # get the list of the tests to execute
   iret, list_tests = get_list(ltest)
   if iret != 0:
      run.Mess(_('error during reading file : %s') % ltest, _('<F>_ERROR'))
   nbtest = len(list_tests)
   
   # print a summary
   summary = _("""
--- %d test-cases to run
--- Run started at %s
--- Parameters used for this run :
    Directory of reference files    : %s
    Directory of developper files   : %s
    Directory to copy results       : %s
    Code_Aster version              : %s
    Executable filename             : %s
    Commands catalogue directory    : %s
    Elements catalogue filename     : %s
    Maximum number of errors (NOOK) : %d
    Criteria to copy result files   : %s
    Time multiplicative factor      : %f
""")
   sum_end = _("""
--- All tests finished at %s

--- Results :
""")
   info_start = ( nbtest, time.strftime('%c'),
      os.path.join(REPREF, conf['SRCTEST'][0]), ', '.join(reptest), resutest,
      prof['version'][0], kargs['exe'], kargs['cmde'], kargs['ele'],
      nbmaxnook, cpresok, facmtps )
   print summary % info_start

   # remove astout profile
   os.remove('%s.export' % jn)

   # 3.6.2. for each test of ltest
   fmt_res0 = 'testcase     result                cpu      sys    cpu+sys  elapsed'
   fmt_resu = '%-12s %-18s %8.2f %8.2f %8.2f %8.2f'
   fmt_res2 = '%4d tests   %4d errors        %8.2f %8.2f %8.2f %8.2f'
   fmt_tot = '-'*12+' '+'-'*18+' '+'-'*8+' '+'-'*8+' '+'-'*8+' '+'-'*8
   lresu = []
   nbnook = 0
   i = 0
   for test in list_tests:
      i += 1
      run.ReinitDiag('OK')
      tit = _('Running test case : %s (%d/%d)...') % (test, i, nbtest)
      run.Mess(tit, 'TITLE')
      
      # copy datas
      ptest = prof.copy()
      ptest.data = []
      ptest.resu = []
      ptest.filename = '%s.export' % test
      copytestD(run, conf, test, REPREF, reptest, reptrav, ptest)
      fpara = os.path.join(reptrav, '%s.para' % test)
      
      # write export file in current directory
      iret, dico, msg = getpara(fpara, run['plate-forme'])
      if iret == 0:
         ptest.args['memjeveux']      = dico['memjeveux']
         ptest.args['memjeveux_stat'] = dico['memjeveux_stat']
         ptest.args['tpmax']          = dico['tps_job']
         ptest['mem_aster']           = dico['mem_aster']
      else:
         run.Mess(msg, _('<E>_PARA_FILE'))
      ptest['actions'] = 'make_etude'
      ptest.WriteExportTo(ptest.filename)
      
      # execute the test
      run.timer.Start(test, hide=True)
      diag, tcpu, tsys, ttot, validbase = execute(
         reptrav,
         multiple       = True,
         with_dbg       = False,
         only_env       = False,
         follow_output  = False or run['debug'],
         fpara          = fpara,
         facmtps        = facmtps,
         cas_test       = True,
         run=run, conf=conf, prof=prof, build=build, exe=kargs['exe'])
      cpu_test, sys_test, treel = run.timer.StopAndGet(test)
      lresu.append([test, diag, tcpu, tsys, ttot, treel])
      print fmt_resu % tuple(lresu[-1])

      # 3.6.2.9. copy results
      tit = _('Copying results')
      run.timer.Start(tit)
      if cpresok == 'RESOK' or \
        (cpresok == 'RESNOOK' and run.GetGrav(diag) > run.GetGrav('<A>')):
         copytestR(run, test, reptrav, resutest)
      run.timer.Stop(tit)

      # count nook
      if run.GetGrav(diag) >= run.GetGrav('NOOK'):
         nbnook += 1
      if nbnook >= nbmaxnook:
         run.Mess(_('Maximum number of errors reached : %d') % nbmaxnook,
               _('<E>_ERROR'))
         break

      # clear reptrav for next execution
      # delete all but not exe, ele, REPPY + ltest
      lf = glob('*')
      for k in (os.path.basename(kargs['exe']), 'elem.1', conf['REPPY'][0],
                os.path.basename(ltest),
                'REPE_OUT', 'RESU_ENSIGHT'):
         try:
            lf.remove(k)
         except ValueError:
            pass
      for f in lf:
         run.Delete(f)

   # Summary
   tit = _('Summary of the run')
   run.Mess(tit, 'TITLE')
   print summary % info_start
   print sum_end % time.strftime('%c')
   t = [0., 0., 0., 0.]
   print fmt_res0
   print fmt_tot
   lresu.sort()
   for lin in lresu:
      t[0] += lin[2]
      t[1] += lin[3]
      t[2] += lin[4]
      t[3] += lin[5]
      print fmt_resu % tuple(lin)
   print fmt_tot
   print fmt_res2 % (i, nbnook, t[0], t[1], t[2], t[3])
   # force global diagnostic to <F>_ERROR if errors occured
   if nbnook > 0:
      run.diag = _('<E>_ERROR')
   run.CheckOK()
   os.chdir(prev)

#-------------------------------------------------------------------------------
def copytestD(run, conf, test, REPREF, reptest, reptrav, ptest):
   """Copy needed files to run 'test' in 'reptrav' : First take files from
   'REPREF' and eventually complete with files from 'reptest'.
   Fill the ASTER_PROFIL `ptest`.
   """
   REPTEST = os.path.join(REPREF, conf['SRCTEST'][0])
   if run.IsRemote(REPTEST):
      run.Mess(_('REPREF (%s) should be on exec host') % REPTEST,
            _('<F>_INVALID_DIR'))
   else:
      REPTEST = run.PathOnly(REPTEST)
   dtyp = dict_typ_test(test)
   for v in dtyp.values():
      run.Delete(v[0])

   # copy files from REPTEST and 'reptest' if not empty
   lrep = [REPTEST]
   if reptest:
      lrep.extend(reptest)
   for src in lrep:
      for typ in dtyp.keys() + ['com?', '[0-9]*']:
         f = os.path.join(src, '%s.%s' % (test, typ))
         if not typ in dtyp.keys() or run.Exists(f):
            run.Copy(reptrav, f, niverr='SILENT')

   lf = glob(os.path.join(reptrav, test)+'.*')
   lf.sort()
   icomm = 0
   for f in lf:
      suff = f.split('.')[-1]
      typ = suff
      dico = {}
      if suff in dtyp.keys() or suff.isdigit():
         dest = 'fort.' + suff
         if suff in dtyp.keys():
            dest, suff = dtyp[suff]
         else:
            typ = 'libr'
         if run.Exists(dest) and os.path.basename(f) != dest:
            run.Mess(_("'%s' overwrites '%s'") % (os.path.basename(f), dest),
                     _('<A>_COPY_DATA'))
         dico = {
            'type'  : typ,
            'path'  : os.path.join(reptrav, dest),
            'isrep' : typ in ('ensi', 'repe'),
            'ul'    : suff,
            'compr' : False,
         }
         run.Rename(f, os.path.join(reptrav, dest))
      else:
         mat = re.search('com([0-9]+)', suff)
         if mat:
            icomm += 1
            fdest = os.path.join(reptrav, 'fort.1.%d' % icomm)
            run.Rename(f, fdest)
            dico = {
               'type'  : 'comm',
               'path'  : f,
               'isrep' : False,
               'ul'    : 1,
               'compr' : False,
            }
      if dico:
         ptest.Set('D', dico)

#-------------------------------------------------------------------------------
def copytestR(run, test, reptrav, resutest):
   """Copy standard results of a test from 'reptrav' to 'resutest'.
   """
   # supported file types
   dtyp = { 'fort.6' : 'mess', 'fort.8' : 'resu', 'fort.9' : 'erre', 'fort.15' : 'code' }
   for f in dtyp.keys():
      fic = os.path.join(reptrav, f)
      dest = os.path.join(resutest, test+'.'+dtyp[f])
      run.Delete(dest)
      if os.path.exists(fic):
         run.Copy(dest, fic, niverr=_('<A>_COPY_TEST'))



