#!/usr/bin/env python

import os
import sys
import re

# ----- get current location
main = sys.argv[0]
if os.path.islink(main):
   main = os.path.realpath(main)
aster_root = os.path.normpath(os.path.join(
      os.path.dirname(os.path.abspath(main)), os.pardir, os.pardir, os.pardir))

if aster_root.endswith('SRC'):
   aster_root = '/opt/aster'

# ----- get default version
fconfig = os.path.join(aster_root, 'ASTK', 'ASTK_SERV', 'conf', 'config')
txt = open(fconfig, 'r').read()
mat = re.search('default_vers *: *([A-Z0-9\.]+)', txt)
assert mat != None, 'default_vers not found in %s' % fconfig
aster_version = mat.group(1)

# ----- write file
content = r"""# Generated by config.py
ASTER_ROOT     = %s
ASTER_VERSION ?= %s
TOOLS_DIR      = $(ASTER_ROOT)/outils

as_run        ?= $(TOOLS_DIR)/as_run

# config file
CONFIG        ?=
ifneq ($(CONFIG),)
CONFIG_FILE_LINE = F conf $(CONFIG) D 0
endif

# common parts
PWD := $(shell pwd)


.PHONY: diag
diag: unittest
	@echo "   >>> Unittest ended with `cat unittest` for $(TEST)"

%%.export: %%.export.in
	@sed -e 's@ASTER_ROOT@$(ASTER_ROOT)@g' \
	     -e 's@ASTER_VERSION@$(ASTER_VERSION)@g' \
	     -e 's@PWD@$(PWD)@g' \
	     -e 's@CONFIG_FILE_LINE@$(CONFIG_FILE_LINE)@g' \
	   $< > $@

%%.py: %%.py.in
	@sed -e 's@ASTER_ROOT@$(ASTER_ROOT)@g' \
	     -e 's@ASTER_VERSION@$(ASTER_VERSION)@g' \
	     -e 's@PWD@$(PWD)@g' \
	     -e 's@CONFIG_FILE_LINE@$(CONFIG_FILE_LINE)@g' \
	   $< > $@
	@chmod 755 $@

.PHONY: clean
clean:
ifneq ($jobclean,)
	@if [ -f jobid ]; then \
	   rm -f $(HOME)/flasheur/*.?`cat jobid`; \
	fi
endif
	@rm -rf output.* error.* exit_code unittest $(TODELETE)

"""
open('Makefile.inc', 'w').write(content % (aster_root, aster_version))



#.PHONY: exec_as_run
#exec_as_run:
#	$(as_run) $(as_run_args) > result 2> error
#	@grep EXIT_CODE result > exit_as_run

#define execute:
#	$(as_run) $^ > result 2> error
#	@grep EXIT_CODE result > exit_code
#endef


